package com.zerog.ia.customcode.actions;

import java.io.*;
import java.util.*;
import java.lang.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;

import com.zerog.ia.api.pub.*;
import com.zerog.awt.ZGStandardDialog;

import com.zerog.ia.customcode.utils.OtherUtils;

/**
 * EndProcess.java
 *
 * This custom code action allows you to kill a running process
 *
 */

public class EndProcess extends CustomCodeAction
{
    // set initial text for dialog displayed if process found
    private static String dialogTitle = "Warning";
    private static String dialogText = "Warning";
    private static String dialogButton1 = "Next";
    private static String dialogButton2 = "Cancel";

    public void install(InstallerProxy ip) throws InstallException
    {
        // get value IA variables
        // executetime should be set to either install or uninstall
        String executetime = ip.substitute("$EP_EXECUTE_TIME$");
        // command that will be passed to pv.exe
        String command = ip.substitute("$EP_COMMAND$");
        // displayDialog is boolean that will determine if a confirmation dialog will be displayed
        String displayDialog = ip.substitute("$EP_DISPLAY_DIALOG$");
        // dialogNarrativeText contains the text to be displayed in the confirmation dialog
        String dialogNarrativeText = ip.substitute("$EP_DIALOG_TEXT$");


        //by default $EP_DIALOG_TEXT$ is set to "notset"
        if(dialogNarrativeText.equals(""))
            dialogNarrativeText = "notset";

        //by default $EP_DISPLAY_DIALOG$ is set to "true"
        if(displayDialog.equals(""))
            displayDialog = "true";

        if(command.equals(""))
        {
            System.err.println("\nEndProcess: $EP_COMMAND$ not set.\n");
        }
        //execute during install
        if(executetime.equalsIgnoreCase("install"))
        {
            try
            {
                URL sourceURL = ip.getResource("/resources/pv.exe");
                String tempdir = ip.getTempDirectory().getAbsolutePath();
                //execute the command specified by
                String EP_ExitCode = processCommand(sourceURL, tempdir, executetime, command, displayDialog, dialogNarrativeText);

                if(EP_ExitCode.equals("exit"))
                {
                    ip.abortInstallation(0);
                }
                else  //set $EP_OUTPUT$ with current list of running processes
                {
                    ip.setVariable("EP_OUTPUT", EP_ExitCode);
                }
            }
            catch(IOException ioe)
            {
                System.err.println("End Process: ");
                ioe.printStackTrace();
            }
        }
        //execute during uninstall
        else if(executetime.equalsIgnoreCase("uninstall"))
        {
            try
            {
                //write variables to DataOutput for use during uninstall
                DataOutput myDataOutput  = ip.getLogOutput();
                myDataOutput.writeUTF(executetime);
                myDataOutput.writeUTF(command);
                myDataOutput.writeUTF(displayDialog);
                myDataOutput.writeUTF(dialogNarrativeText);

            }
            catch(Exception e)
            {
                System.err.println("End Process: ");
                e.printStackTrace();
            }
        }
        else
        {
            System.err.println("\nEndProcess: $EP_EXECUTE_TIME$ is not set correctly.");
            System.err.println("$EP_EXECUTE_TIME$: " + executetime + "\n");
        }
    }

	public void uninstall(UninstallerProxy up)
	{

		DataInput din = up.getLogInput();

		try
		{
            //read in IA variables via DataInput
            String executetime = din.readUTF();
			String proccmd = din.readUTF();
            String displayD = din.readUTF();
            String dialognar = din.readUTF();

            //prep values of IA variables
			proccmd.trim();
            executetime = executetime.toLowerCase();
            displayD = displayD.toLowerCase();

            URL sourceURL = up.getResource("/resources/pv.exe");
            String tempdir = up.getTempDirectory().getAbsolutePath();
            processCommand(sourceURL, tempdir, executetime, proccmd, displayD, dialognar);

        }
        catch(Exception e)
        {
            System.err.println("\nEnd Process: ");
            e.printStackTrace();
        }
	}

    // This method will extract pv.exe to a temp location and use it to execute the command specified by $EP_COMMAND$
    public String processCommand(URL sourceURL, String tempdir, String executetime, String proccmd, String displayD, String dialognar)
    {

       String exitCode = null;
       try
       {
           StringTokenizer stp = new StringTokenizer(proccmd);
           int numofargs = stp.countTokens();

           String proc = "";
           Vector v = new Vector();

           //tokenize string to get process
           while(stp.hasMoreTokens())
           {
               v.add(stp.nextToken());
           }

           //gets the options and process to execute
           proc = (String)v.lastElement();
           String options = proccmd.substring(0, proccmd.indexOf(proc));

           // System.err.println("sourceURL: " + sourceURL);

           File newProcessCmd = new File(tempdir + "\\pv.exe");

           // System.err.println("newProcessCmd: " + newProcessCmd);

           // extract pv.exe to temp location.
           com.zerog.ia.customcode.utils.OtherUtils.extractResource( sourceURL, newProcessCmd.getAbsolutePath() );
           String executeString = "\"" + newProcessCmd + "\" " + options + proc;

           //execute pv.exe to get a list of all running processes.
           final Process p = Runtime.getRuntime().exec(newProcessCmd.getAbsolutePath());
           final StringWriter errWriter = new StringWriter();
           final StringWriter outWriter = new StringWriter();

           Thread thread = new Thread()
           {
               public void run()
               {
                   InputStream err = p.getErrorStream();
                   int c;
                   try
                   {
                       while ((c = err.read()) >= 0)
                       {
                           errWriter.write(c);
                           System.err.println(errWriter.toString());
                       }
                   }
                   catch (IOException ioe)
                   {
                       System.err.println("End Process: ");
                       ioe.printStackTrace();
                   }
               }
           };

           Thread thread2 = new Thread()
           {
               public void run()
               {
                   InputStream err = p.getInputStream();
                   int c;
                   try
                   {
                       while ((c = err.read()) >= 0)
                       {
                           outWriter.write(c);
                       }
                   }
                   catch (IOException ioe)
                   {
                       System.err.println("End Process: ");
                       ioe.printStackTrace();
                   }
               }
           };

           thread.start();
           thread2.start();
           p.waitFor();

           String stdErr   = errWriter.toString();
           String stdOut   = outWriter.toString();
           exitCode = new Integer(p.exitValue()).toString();

           StringTokenizer st = new StringTokenizer(stdOut, "\n");
           int pcounter = 0;
           String currentProcess;

           //tokenize the output from pv.exe to determine if the specifiec process is running
           while (st.hasMoreTokens())
           {
               StringTokenizer st1 = new StringTokenizer(st.nextToken());
               if(st1.hasMoreTokens())
               {
                   currentProcess = st1.nextToken();
                   if (currentProcess.equals(proc))
                   {
                       pcounter++;
                   }
               }

           }

           // execute command if 1 or more of the specified process is running
           if(pcounter>0)
           {
               System.err.println("There are " + pcounter + " instances of " + proc + " currently running.");
               String narrative = "";
               int buttonpressed = 0;
               //if $EP_DISPLAY_DIALOG$ is set to true, display dialog
               if(displayD.equals("true"))
               {
                   if(dialognar.equals("notset"))
                   {
                       narrative = "The following process is currently running.\n\n" + proc + "\n\nSelect \"Next\" to shutdown this process and continue or select \"Cancel\" to exit the " + executetime + ".";
                   }
                   else
                   {
                       narrative = dialognar;
                   }

                   // create and display dialog
                   ZGStandardDialog zgd = new ZGStandardDialog(dialogTitle, dialogText, narrative);
                   zgd.setDefaultButtonLabel(dialogButton1);
                   zgd.setCancelButtonLabel(dialogButton2);
                   zgd.setCancelButtonVisible(true);
                   zgd.setAlertLevel(ZGStandardDialog.WARNING);
                   zgd.setModal(true);
                   zgd.show();

                   buttonpressed = zgd.getLastButtonPressed();
               }
               //if the user selects to continue, or if $EP_DISPLAY_DIALOG$ is false, execute command
               if((buttonpressed==1)||(displayD.equals("false")))
               {
                   System.err.println(dialogButton1);
                   System.err.println("EndProcess Command: " + executeString);
                   Process p1 = Runtime.getRuntime().exec(executeString);
                   p1.waitFor();
                   System.err.println("EndProcess exit code: " + p1.exitValue());
                   exitCode = Integer.toString(p1.exitValue());
               }
               // exit uninstall
               else  if((buttonpressed==2))
               {
                   System.err.println(dialogButton1);
                   if(executetime.equals("install"))
                   {
                       exitCode = "exit";
                   }
                   if(executetime.equals("uninstall"))
                   {
                       System.exit(0);
                   }
               }
               else
               {
                   System.err.println("End Process: ERROR");
               }
           }
           else
           {
               // no processes found
               exitCode = Integer.toString(1);
           }
       }
       catch(Exception e)
       {
           System.err.println("End Process: ");
           e.printStackTrace();
       }

       return exitCode;
   }

    public String getInstallStatusMessage()
    {
        return "End Process Action";
    }

    public String getUninstallStatusMessage()
    {
        return "End Process Action";
	}

}
